@@ -79,6 +79,8 @@ def likelihood_fit(**kwargs):
         log_path = os.path.splitext(_kwargs["outname"])[0] + ".log"
         with standard_log(log_path) as logger:
             analysis = AnalysisBase(**_init_kwargs)
+            if _kwargs['export_as_np_pulls']:
+                analysis.minimizer.configure(hesse=True)
             fit_result = analysis.nll_fit(mode=3)
         print(f"INFO: Saved fit log to `{log_path}`")
     else:
