@@ -8,10 +8,9 @@ import sys
 import copy
 import math
 import fnmatch
-from typing import List, Optional, Union, Dict, Set
+from typing import List, Optional, Union, Dict, Set, Tuple
 
 import numpy as np
-import pandas as pd
 
 import ROOT
 
@@ -22,7 +21,9 @@ from quickstats.maths.numerics import is_integer, pretty_value
 from quickstats.utils.root_utils import load_macro
 from quickstats.utils.common_utils import str_list_filter
 from quickstats.utils.roofit_utils import get_variable_attributes, variable_collection_to_dataframe
-from quickstats.interface.root import TH1, RooDataSet
+
+from quickstats.interface.root import TH1, RooDataSet, RooArgSet, RooMsgService
+from quickstats.components.basics import WSVariables, SetValueMode
 
 class ExtendedModel(AbstractObject):
     
@@ -110,7 +111,11 @@ class ExtendedModel(AbstractObject):
     
     @semistaticmethod
     def load_extension(self):
-        for extension in ['RooTwoSidedCBShape', 'FlexibleInterpVarMkII']:
+        if (quickstats.root_version >= (6, 26, 0)):
+            extensions = ['RooTwoSidedCBShape']
+        else:
+            extensions = ['RooTwoSidedCBShape', 'FlexibleInterpVarMkII']
+        for extension in extensions:
             result = load_macro(extension)
             if (result is not None) and hasattr(ROOT, extension):
                 self.stdout.info(f'INFO: Loaded extension module "{extension}"')
@@ -131,14 +136,7 @@ class ExtendedModel(AbstractObject):
                     self.stdout.info('INFO: {} {} interpolation code set to {}'.format(component.GetName(),
                                                                             class_name,
                                                                             interp_code))
-        return None
-    
-    @staticmethod
-    def remove_msg_topic(input_arguments=True, numeric_integration=True):
-        if input_arguments:
-            ROOT.RooMsgService.instance().getStream(1).removeTopic(ROOT.RooFit.InputArguments)
-        if numeric_integration:
-            ROOT.RooMsgService.instance().getStream(1).removeTopic(ROOT.RooFit.NumIntegration)
+        return None           
 
     @semistaticmethod
     def activate_binned_likelihood(self, ws):
@@ -253,7 +251,7 @@ class ExtendedModel(AbstractObject):
             self.data_name = data_names[0]
         data = ws.data(self.data_name)
         if not data:
-            raise RuntimeError(f'Failed to load dataset \"{self.data_name}\"')
+            raise RuntimeError(f'failed to load dataset \"{self.data_name}\"')
         self.stdout.info('INFO: Loaded dataset "{}" from workspace'.format(data.GetName()))
                 
         # load nuisance parameters
@@ -300,64 +298,48 @@ class ExtendedModel(AbstractObject):
         # Load snapshots
         self.load_snapshots(self.initial_snapshots)
         
-        self.remove_msg_topic()
+        RooMsgService.remove_topics()
         return None
                 
-    @semistaticmethod
-    def _fix_parameters(self, source:"ROOT.RooArgSet", param_expr=None, param_str='parameter'):
-        '''
-            source: parameters instance
-            param_expr: 
-        '''
+    def set_parameters_with_expression(self, param_expr:Optional[str]=None,
+                                       source:Optional["ROOT.RooArgSet"]=None,
+                                       mode:Union[str, SetValueMode]=SetValueMode.UNCHANGED):
+        if source is None:
+            source = self.workspace.allVars()
         param_dict = self.parse_param_expr(param_expr)
-        return self._set_parameters(source, param_dict, mode='fix', param_str=param_str)           
+        parameters = self._set_parameters(source, param_dict, mode=mode)
+        if not parameters:
+            self.stdout.warning("WARNING: No parameters are modified from the given fix / profile expression", "red")
+        return parameters
+        
+    def fix_parameters(self, param_expr:Optional[str]=None, source:Optional["ROOT.RooArgSet"]=None):
+        return self.set_parameters_with_expression(param_expr, source, mode=SetValueMode.FIX)
     
-    @semistaticmethod
-    def _profile_parameters(self, source:"ROOT.RooArgSet", param_expr=None, param_str='parameter'):
-        '''
-            source: parameters instance
-            param_expr: 
-        '''                          
-        param_dict = self.parse_param_expr(param_expr)
-        return self._set_parameters(source, param_dict, mode='free', param_str=param_str)   
+    def profile_parameters(self, param_expr:Optional[str]=None, source:Optional["ROOT.RooArgSet"]=None):
+        return self.set_parameters_with_expression(param_expr, source, mode=SetValueMode.FREE)
     
-    def fix_parameters(self, param_expr=None):
-        return self._fix_parameters(self.workspace.allVars(), param_expr=param_expr,
-                                    param_str='parameter')
-    
-    def profile_parameters(self, param_expr=None):
-        profiled_parameters = self._profile_parameters(self.workspace.allVars(), param_expr=param_expr,
-                                                       param_str='parameter') 
-        if not profiled_parameters:
-            self.stdout.info('Info: No parameters are profiled.')
-        return profiled_parameters 
-    
-    def fix_nuisance_parameters(self, param_expr=None):
-        return self._fix_parameters(self.nuisance_parameters, param_expr=param_expr,
-                                    param_str='nuisance parameter')
-                          
-    def fix_parameters_of_interest(self, param_expr=None):
-        return self._fix_parameters(self.pois, param_expr=param_expr, param_str='parameter of interest')
-
-    def profile_parameters_of_interest(self, param_expr=None):
-        return self._profile_parameters(self.pois, param_expr=param_expr, param_str='parameter of interest')
-    
-    @semistaticmethod
-    def _set_parameters(self, source:"ROOT.RooArgSet", param_dict, mode=None, param_str='parameter'):
-        set_parameters = []
-        available_parameters = [param.GetName() for param in source]
+    def _set_parameters(self, source:"ROOT.RooArgSet", param_dict:Dict, 
+                        mode:Union[str, SetValueMode]=SetValueMode.UNCHANGED):
+        selected_parameters = []
+        source_names = [param.GetName() for param in source]
         for name in param_dict:
-            selected_params = [param for param in available_parameters if fnmatch.fnmatch(param, name)]
+            # resolve variable set
+            if name.startswith("<") and name.endswith(">"):
+                selected_params = self.get_variables(name.strip("<>"))
+            else:
+                selected_params = [sname for sname in source_names if fnmatch.fnmatch(sname, name)]
             if not selected_params:
-                self.stdout.warning('WARNING: Parameter "{}" does not exist. No modification will be made.'.format(name))
-            for param_name in selected_params:
-                self._set_parameter(source[param_name], param_dict[name], mode=mode, param_str=param_str)
-                set_parameters.append(source[param_name])
+                self.stdout.warning(f'WARNING: Parameter "{name}" does not exist. No modification will be made.', "red")
+            for param in selected_params:
+                if isinstance(param, str):
+                    param = source[param]
+                self._set_parameter(param, param_dict[name], mode=mode)
+                selected_parameters.append(param)
+        return selected_parameters
 
-        return set_parameters
-    
-    @semistaticmethod
-    def _set_parameter(self, param, value, mode=None, param_str='parameter'):
+    def _set_parameter(self, param:"ROOT.RooRealVar", value:Union[float, int, List, Tuple], 
+                       mode:Union[str, SetValueMode]=SetValueMode.UNCHANGED):
+        mode = SetValueMode.parse(mode)
         name = param.GetName()
         old_value = param.getVal()
         new_value = old_value
@@ -376,7 +358,6 @@ class ExtendedModel(AbstractObject):
                 if (new_value < v_min) or (new_value > v_max):
                     new_value = (v_min + v_max)/2
                 param.setRange(v_min, v_max)
-                self.stdout.info('INFO: Set {} "{}" range to ({},{})'.format(param_str, name, v_min, v_max))
             elif (v_min is not None):
                 if (new_value < v_min):
                     new_value = v_min
@@ -384,30 +365,29 @@ class ExtendedModel(AbstractObject):
                 if (v_min == 0) and (old_value < 0):
                     new_value = abs(old_value)
                 param.setMin(v_min)
-                self.stdout.info('INFO: Set {} "{}" min value to ({},{})'.format(param_str, name, v_min))
             elif (v_max is not None):
                 if (new_value > v_max):
                     new_value = v_max
-                # upper bound is zero, if original value is positive, will flip to negative value
                 if (v_max == 0) and (old_value > 0):
-                    new_value = -abs(old_value)                    
+                    new_value = -abs(old_value)
                 param.setMax(v_max)
-                self.stdout.info('INFO: Set {} "{}" max value to ({},{})'.format(param_str, name, v_max))
         if new_value != old_value:
-            param.setVal(new_value)              
-            self.stdout.info('INFO: Set {} "{}" value to {}'.format(param_str, name, new_value))
-        if mode=='fix':
+            param.setVal(new_value)
+        if mode == SetValueMode.FIX:
             param.setConstant(1)
-            self.stdout.info('INFO: Fixed {} "{}" at value {}'.format(param_str, name, param.getVal()))
-        elif mode=='free':
+            self.stdout.info(f"INFO: Fixed parameter \"{name}\" at value {param.getVal()} "
+                             f"[{param.getMin()}, {param.getMax()}]")
+        elif mode == SetValueMode.FREE:
             param.setConstant(0)
-            self.stdout.info('INFO: "{}" = [{}, {}]'.format(name, param.getMin(), param.getMax()))
-        return None
-
-    @staticmethod
-    def set_parameter_defaults(source:"ROOT.RooArgSet", value=None, error=None, constant=None,
-                               remove_range=None, target:List[str]=None):
+            self.stdout.info(f"INFO: Float parameter \"{name}\" at value {param.getVal()} "
+                             f"[{param.getMin()}, {param.getMax()}]")
+        else:
+            state_str = "C" if param.isConstant() else "F"
+            self.stdout.info(f"INFO: Set parameter \"{name}\" at value {param.getVal()} "
+                             f"[{param.getMin()}, {param.getMax()}] {state_str}")
 
+    def set_parameter_defaults(source:"ROOT.RooArgSet", value:float=None, error:float=None,
+                               constant:bool=None, remove_range:bool=None, target:List[str]=None):
         for param in source:
             if (not target) or (param.GetName() in target):
                 if remove_range:
@@ -418,10 +398,9 @@ class ExtendedModel(AbstractObject):
                     param.setError(error)
                 if constant is not None:
                     param.setConstant(constant)
-        return None
     
     @staticmethod
-    def parse_param_expr(param_expr):
+    def parse_param_expr(param_expr:str=None):
         param_dict = {}
         # if parameter expression is not empty string or None
         if param_expr: 
@@ -439,7 +418,7 @@ class ExtendedModel(AbstractObject):
     @staticmethod
     def parse_param_str(param_str):
         '''
-        Example: "param_1,param_2=0.5,param_3=-1,param_4=1,param_5=0:100,param_6=:100,param_7=0:"
+        Example: "param_1,param_2=0.5,param_3=-1,param_4=1,param_5=_0_100,param_6=__100,param_7=_0_"
         '''
         param_str = param_str.replace(' ', '')
         param_list = param_str.split(',')
@@ -589,10 +568,13 @@ class ExtendedModel(AbstractObject):
         nuis_list, glob_list, _ = self._pair_constraints(constraint_set, self.global_observables, self.nuisance_parameters)
         return nuis_list, glob_list
 
-    def pair_constraints(self, to_str=False, base_component:bool=False):
+    def pair_constraints(self, to_str=False, base_component:bool=False, sort:bool=True):
         constraint_set = self.unfold_constraints()
+        nuisance_parameters = self.nuisance_parameters
+        if sort:
+            nuisance_parameters = RooArgSet.sort(nuisance_parameters)
         nuis_list, glob_list, pdf_list = self._pair_constraints(constraint_set, self.global_observables, 
-                                                                self.nuisance_parameters,
+                                                                nuisance_parameters,
                                                                 base_component=base_component)
         if to_str:
             nuis_names = [i.GetName() for i in nuis_list]
@@ -688,7 +670,7 @@ class ExtendedModel(AbstractObject):
                             if math.isclose(old_sigma_value, 1.0, abs_tol=0.001):
                                 old_sigma_value = 1.0
                             if not found_sigma:
-                                self.stdout.info('INFO: Sigma for pdf {} not found. Uisng 1.0.'.format(constr_name))
+                                self.stdout.info(f'INFO: Sigma for pdf {constr_name} not found. Uisng 1.0.')
                             else:
                                 self.stdout.info('INFO: Uisng {} for sigma of pdf {}'.format(old_sigma_value, constr_name))
 
@@ -696,11 +678,11 @@ class ExtendedModel(AbstractObject):
                         elif constraint.IsA() == ROOT.RooPoisson.Class():
                             constraint_type = 'pois'
                             tau = glob_obs.getVal()
-                            self.stdout.info('INFO: Found tau {} of pdf'.format(constr_name))
+                            self.stdout.info(f'INFO: Found tau {constr_name} of pdf')
                             prefit_variation = 1. / math.sqrt(tau)
-                            self.stdout.info('INFO: Prefit variation is {}'.format(prefit_variation))
+                            self.stdout.info(f'INFO: Prefit variation is {prefit_variation}')
                             nuip_nom = 1.0
-                            self.stdout.info("INFO: Assume that {} is nominal value of the nuisance parameter".format(nuip_nom))
+                            self.stdout.info(f"INFO: Assume that {nuip_nom} is nominal value of the nuisance parameter")
         return prefit_variation, constraint_type, nuip_nom
     
     def create_blind_range(self, blind_range:List[float], categories:Optional[List[str]]=None):
@@ -807,7 +789,7 @@ class ExtendedModel(AbstractObject):
             nuis_name = nuis.GetName()
             prefit_variation, constraint_type, _ = self.inspect_constrained_nuisance_parameter(nuis, all_constraints)
             if constraint_type=='gaus':
-                self.stdout.info('INFO: Changing error of {} from {} to {}'.format(nuis_name, nuis.getError(), prefit_variation))
+                self.stdout.info(f'INFO: Changing error of {nuis_name} from {nuis.getError()} to {prefit_variation}')
                 nuis.setError(prefit_variation)
                 nuis.removeRange()    
         return None
@@ -860,7 +842,7 @@ class ExtendedModel(AbstractObject):
         elif isinstance(obs_and_weight, str):
             obs_and_weight = self.workspace.set(obs_and_weight)
             if not obs_and_weight:
-                raise RuntimeError('named set "{}" not found in workspace'.format(obs_and_weight))
+                raise RuntimeError(f'named set "{obs_and_weight}" not found in workspace')
         elif not isinstance(obs_and_weight, ROOT.RooArgSet):
             raise ValueError('the argument "obs_and_weight" must be of RooArgSet type')
         return obs_and_weight, weight_var
@@ -872,7 +854,7 @@ class ExtendedModel(AbstractObject):
     def get_object_map(object_dict:Dict, object_name:str):
         ExtendedModel.load_RooFitObjects()
         if object_name not in ["RooDataSet", "RooAbsPdf"]:
-            raise ValueError("unsupported object \"{}\"".format(object_name))
+            raise ValueError(f"unsupported object \"{object_name}\"")
         object_map = ROOT.std.map(f"string, {object_name}*")()
         object_map.keepalive = list()
         for c, d in object_dict.items():
@@ -1093,7 +1075,7 @@ class ExtendedModel(AbstractObject):
             else:
                 raise ValueError(f"unsupported constraint option: {constraint_option}")
             if isinstance(nll_options, dict):
-                minimizer.configure_nll(None, all_globs, **nll_options)
+                minimizer.configure_nll(**nll_options)
             elif isinstance(nll_options, list):
                 minimizer.set_nll_commands(nll_options)
             else:
@@ -1257,33 +1239,6 @@ class ExtendedModel(AbstractObject):
             raise RuntimeError('Failed to load model config "{}"'.format(mc_name))
         return file, ws, mc
     
-    @staticmethod
-    def get_nuisance_parameter_names(fname:str, ws_name:Optional[str]=None, mc_name:Optional[str]=None):
-        ExtendedModel.load_extension()
-        file, ws, mc = ExtendedModel.load_ws(fname, ws_name, mc_name)
-        nuisance_parameters = mc.GetNuisanceParameters()
-        if not nuisance_parameters:
-            raise RuntimeError('Failed to load nuisance parameters')
-        return [nuis.GetName() for nuis in nuisance_parameters]
-    
-    @staticmethod
-    def get_poi_names(fname:str, ws_name:Optional[str]=None, mc_name:Optional[str]=None):
-        ExtendedModel.load_extension()
-        file, ws, mc = ExtendedModel.load_ws(fname, ws_name, mc_name)
-        pois = mc.GetParametersOfInterest()
-        if not pois:
-            raise RuntimeError('Failed to load parameters of interest')        
-        return [poi.GetName() for poi in pois]
-
-    @staticmethod
-    def get_dataset_names(fname:str, ws_name:Optional[str]=None, mc_name:Optional[str]=None):
-        ExtendedModel.load_extension()
-        file, ws, mc = ExtendedModel.load_ws(fname, ws_name, mc_name)
-        datasets = ws.allData()
-        if not datasets:
-            raise RuntimeError('Failed to load datasets')
-        return [dataset.GetName() for dataset in datasets]
-    
     def get_category_map(self, pdf=None):
         if pdf is None:
             pdf = self.pdf
@@ -1581,31 +1536,60 @@ class ExtendedModel(AbstractObject):
             pdf.close()
         return None
     
-    def as_dataframe(self, variable_type:str):
-        if variable_type == "pois":
-            vcollection = self.pois
-        elif variable_type == "nuisance_parameters":
-            vcollection = self.nuisance_parameters
-        elif variable_type == "global_observables":
-            vcollection = self.global_observables
-        elif variable_type == "constrained_nuisance_parameters":
-            vcollection = self.get_constrained_nuisance_parameters()
-        elif variable_type == "unconstrained_nuisance_parameters":
-            vcollection = self.get_unconstrained_nuisance_parameters()
-        elif variable_type == "constraints":
+    def get_variables(self, variable_type:Union[str, WSVariables], sort:bool=True):
+        _variable_type = WSVariables.parse(variable_type)
+        if _variable_type == WSVariables.VARIABLES:
+            variables = self.workspace.allVars()
+        elif _variable_type == WSVariables.OBSERVABLES:
+            variables = self.observables
+        elif _variable_type == WSVariables.POIS:
+            variables = self.pois
+        elif _variable_type == WSVariables.GLOBAL_OBSERVABLES:
+            variables = self.global_observables
+        elif _variable_type == WSVariables.NUISANCE_PARAMETERS:
+            variables = self.nuisance_parameters
+        elif _variable_type == WSVariables.CONSTRAINED_NUISANCE_PARAMETERS:
+            variables = RooArgSet.from_list(self.get_constrained_nuisance_parameters())
+        elif _variable_type == WSVariables.UNCONSTRAINED_NUISANCE_PARAMETERS:
+            variables = RooArgSet.from_list(self.get_unconstrained_nuisance_parameters())
+        elif _variable_type == WSVariables.CONSTRAINTS:
+            nuis_list, glob_list, pdf_list = self.pair_constraints(sort=sort)
+            return RooArgSet.from_list(nuis_list), RooArgSet.from_list(glob_list), RooArgSet.from_list(pdf_list)
+        elif _variable_type == WSVariables.AUXILIARY:
+            variables           = self.get_variables("variables")
+            pois                = self.get_variables("pois")
+            nuisance_parameters = self.get_variables("nuisance_parameters")
+            global_observables  = self.get_variables("global_observables")
+            observables         = self.get_variables("observables")
+            variables.remove(pois)
+            variables.remove(nuisance_parameters)
+            variables.remove(global_observables)
+            variables.remove(observables)
+        else:
+            raise ValueError(f"unknown variable type \"{variable_type}\"")
+        if sort:
+            return RooArgSet.sort(variables)
+        return variables
+    
+    def as_dataframe(self, variable_type:Union[str, WSVariables]):
+        import pandas as pd
+        _variable_type = WSVariables.parse(variable_type)
+        # for the special case of displaying constrain terms
+        # will print the tuple of names of NP, Global Observables and Constrain Pdf
+        if _variable_type == WSVariables.CONSTRAINTS:
             data = self.pair_constraints(to_str=True)
-            df = pd.DataFrame(data, columns =['pdf', 'nuisance parameters', 'global observables'])
+            df = pd.DataFrame(data, columns =['constraint pdf', 'nuisance parameters', 'global observables'])
             return df
         else:
-            raise ValueError("unknown variable type")
+            vcollection = self.get_variables(_variable_type)
         df = variable_collection_to_dataframe(vcollection)
-        return df    
+        return df
     
     def print_summary(self, items:Optional[List]=None, suppress_print:bool=False, detailed:bool=True,
                       include_patterns:Optional[List]=None, exclude_patterns:Optional[List]=None,
                       save_as:Optional[str]=None):
         if items is None:
-            items = ['workspace', 'dataset', 'category', 'observable', 
+            items = ['workspace', 'dataset', 'snapshot', 'category', 'observable', 
                      'poi', 'detailed_nuisance_parameter']
         summary_str = ""
         # workspace
@@ -1617,6 +1601,11 @@ class ExtendedModel(AbstractObject):
             datasets = self.workspace.allData()
             summary_str += f"Datasets ({len(datasets)}):\n"
             summary_str += "".join([f"\t{obs.GetName()}\n" for obs in datasets])
+        # snapshots
+        if ('snapshot' in items) and (quickstats.root_version >= (6, 26, 0)):
+            snapshots = self.workspace.getSnapshots()
+            summary_str += f"Snapshots ({len(snapshots)}):\n"
+            summary_str += "".join([f"\t{snap.GetName()}\n" for snap in snapshots])
         # categories
         if 'category' in items:
             if isinstance(self.pdf, ROOT.RooSimultaneous):
@@ -1628,19 +1617,6 @@ class ExtendedModel(AbstractObject):
                     bin_range = category_map[category]['bin_range']
                     bins = category_map[category]['bins']
                     summary_str += f"\t{category} (observable = {observable}, range = [{bin_range[0]}, {bin_range[1]}], bins = {bins})\n"
-        """
-        # observables
-        if 'observable' in items:
-            summary_str += f"Observables ({len(self.observables)}):\n"
-            for obs in self.observables:
-                if obs.ClassName() != "RooRealVar":
-                    continue
-                obs_name = obs.GetName()
-                bin_range = obs.getRange()
-                n_bin = obs.getBins()
-                obs_str = f"{obs_name} (range = [{bin_range.first}, {bin_range.second}], bins = {n_bin})"
-                summary_str += f"\t{obs_str}\n"
-        """
         # pois, NPs
         param_strs     = []
         param_sets     = []
@@ -1659,6 +1635,9 @@ class ExtendedModel(AbstractObject):
         if 'global_observable' in items:
             param_strs += ['Global Observables']
             param_sets += [self.global_observables]
+        if 'auxiliary' in items:
+            param_strs += ['Auxiliary Parameters']
+            param_sets += [self.get_variables('auxiliary')]            
         for pstr, pset in zip(param_strs, param_sets):
             summary_str += f"{pstr} ({len(pset)}):\n"
             param_names = [p.GetName() for p in pset]
@@ -1684,7 +1663,7 @@ class ExtendedModel(AbstractObject):
                     name = d['name']
                     summary_str += f"\t{name}\n"
         if not suppress_print:
-            print(summary_str)
+            self.stdout.info(summary_str)
         if save_as is not None:
             with open(save_as, "w") as f:
                 f.write(summary_str)
\ No newline at end of file
