@@ -34,6 +34,7 @@ class Likelihood(AnalysisObject):
         config['data_name'] = data_name
         config['verbosity'] = verbosity
         self._inherit_init(super().__init__, **config)
+        self.roofit_result = None
     
     @staticmethod
     def _parse_poi_val(pois:"ROOT.RooArgSet", poi_val:Optional[Union[Dict[str, float], float]]=None):
@@ -79,6 +80,7 @@ class Likelihood(AnalysisObject):
             self.minimizer.minos_set = pois
         
         start_time = time.time()
+        roofit_result = {}
         muhat = {}
         muhat_errlo = {}
         muhat_errhi = {}
@@ -90,6 +92,7 @@ class Likelihood(AnalysisObject):
             self.minimizer.create_nll()
             prefit_nll = self.minimizer.nll.getVal()
             fit_status = self.minimizer.minimize()
+            roofit_result = self.minimizer.fit_result
             self.save_snapshot("nllFit")
             postfit_nll = self.minimizer.nll.getVal()
             fit_time = time.time() - start_time
@@ -106,6 +109,7 @@ class Likelihood(AnalysisObject):
                 val = poi_val[poi.GetName()]
                 poi.setVal(val)
             fit_status_uncond = self.minimizer.minimize()
+            roofit_result['uncond'] = self.minimizer.fit_result
             self.save_snapshot("uncondFit")
             nll_uncond = self.minimizer.nll.getVal()
             for poi in pois:
@@ -126,6 +130,7 @@ class Likelihood(AnalysisObject):
                 val = poi_val.get(poi.GetName(), poi.getVal())
                 poi.setVal(val)
             fit_status_cond = self.minimizer.minimize()
+            roofit_result['cond'] = self.minimizer.fit_result
             self.save_snapshot("condFit")
             nll_cond = self.minimizer.nll.getVal()
         fit_time_cond = time.time() - fit_time_uncond - start_time
@@ -167,6 +172,7 @@ class Likelihood(AnalysisObject):
                 'status': fit_status_cond,
                 'time': fit_time_cond
             }
+            
         if mode == 0:
             pnll = nll_cond - nll_uncond
             qmu  = 2*pnll
@@ -208,6 +214,8 @@ class Likelihood(AnalysisObject):
             fit_result['postfit_nll'] = postfit_nll
             fit_result['total_time'] = fit_time
         
+        self.roofit_result = roofit_result
+        
         # finallizing
         self.minimizer.config['minos'] = tmp_do_minos
         self.minimizer.minos_set = ROOT.RooArgSet()
