@@ -276,7 +276,7 @@ class XMLWSBuilder(AbstractObject):
             
             if self.debug_mode:
                 ws_i.Print()
-        observables.add(channel_list)
+        #observables.add(channel_list)
         self.import_object(self.comb_ws, combined_pdf)
         
         self.comb_ws.defineSet("nuisanceParameters", nuis)
@@ -1581,8 +1581,10 @@ class XMLWSBuilder(AbstractObject):
                 self.import_expression(ws, expr)
             elif name == "ModelItem":
                 factory_str = self._translate_node_attrib(node, "Name", tag_name)
-                pdf_name = factory_str[factory_str.find("::")+2:factory_str.find("(")]
-                factory_str = factory_str.replace(pdf_name, sample.model_name)
+                str_start = factory_str.find("::")+2
+                str_end   = factory_str.find("(")
+                pdf_name = factory_str[str_start:str_end]
+                factory_str = factory_str[:str_start] + sample.model_name + factory_str[str_end:]
                 self.import_expression(ws, factory_str)
                 is_success = True
                 break
