@@ -71,7 +71,7 @@ ROOT_MACROS = \
                 result.push_back(h->GetBinContent(bin_index));
             return result;
         }
-
+      
         template<typename T>
         std::vector<T> GetBinWidthArray(TH1* h, const size_t &underflow_bin=0, const size_t & overflow_bin=0)
         {
@@ -99,6 +99,47 @@ ROOT_MACROS = \
         }    
     }; 
     """,
+    "TAxisUtils": 
+    """
+    namespace TAxisUtils{
+        template<typename T>
+        std::vector<T> GetBinLowEdgeArray(TAxis* ax, const size_t &underflow_bin=0, const size_t & overflow_bin=0)
+        {
+            const size_t n_bin = ax->GetNbins();
+            std::vector<T> result;
+            result.reserve(n_bin);
+            const size_t bin_min = 1 - underflow_bin;
+            const size_t bin_max = n_bin + overflow_bin;
+            for (size_t bin_index = bin_min; bin_index <= bin_max; bin_index++)
+                result.push_back(ax->GetBinLowEdge(bin_index));
+            return result;
+        }
+        template<typename T>
+        std::vector<T> GetBinCenterArray(TAxis* ax, const size_t &underflow_bin=0, const size_t & overflow_bin=0)
+        {
+            const size_t n_bin = ax->GetNbins();
+            std::vector<T> result;
+            result.reserve(n_bin);
+            const size_t bin_min = 1 - underflow_bin;
+            const size_t bin_max = n_bin + overflow_bin;
+            for (size_t bin_index = bin_min; bin_index <= bin_max; bin_index++)
+                result.push_back(ax->GetBinCenter(bin_index));
+            return result;
+        }
+        template<typename T>
+        std::vector<T> GetBinWidthArray(TAxis* ax, const size_t &underflow_bin=0, const size_t & overflow_bin=0)
+        {
+            const size_t n_bin = ax->GetNbins();
+            std::vector<T> result;
+            result.reserve(n_bin);
+            const size_t bin_min = 1 - underflow_bin;
+            const size_t bin_max = n_bin + overflow_bin;
+            for (size_t bin_index = bin_min; bin_index <= bin_max; bin_index++)
+                result.push_back(ax->GetBinWidth(bin_index));
+            return result;
+        }
+    };
+    """,    
     "THistUtils": 
     """
     namespace THistUtils{
