@@ -3,6 +3,7 @@ import os
 import re
 import copy
 import glob
+import fnmatch
 import itertools
 
 import numpy as np
@@ -203,7 +204,9 @@ class ParamParser:
             encoded_str_list.append(encoded_str)
         return "_".join(encoded_str_list)
 
-    def get_external_param_points(self, dirname:str=""):
+    def get_external_param_points(self, dirname:str="",
+                                  filter_expr:Optional[str]=None,
+                                  exclude_expr:Optional[str]=None):
         if os.path.isdir(dirname):
             fnames = glob.glob(os.path.join(dirname, '*'))
         else:
@@ -222,14 +225,74 @@ class ParamParser:
             param_points.append(point)
         attributes = list(self.attribute_parser)
         param_points = self.sort_param_points(param_points, attributes)
-        return param_points
+        selected_param_points = self.select_param_points(param_points, filter_expr,
+                                                         exclude_expr, dict_key="parameters")
+        return selected_param_points
     
-    def get_internal_param_points(self):
-        return self.parse_param_str(self.param_str)
+    def get_internal_param_points(self, filter_expr:Optional[str]=None,
+                                  exclude_expr:Optional[str]=None):
+        param_points = self.parse_param_str(self.param_str)
+        selected_param_points = self.select_param_points(param_points, filter_expr,
+                                                         exclude_expr)
+        return selected_param_points
     
-    def get_param_points(self, dirname:str=""):
-        external_param_points = self.get_external_param_points(dirname)     
-        internal_param_points = self.get_internal_param_points()
+    @staticmethod
+    def parse_filter_expr(expr:Optional[str]=None):
+        if expr is None:
+            return {}
+        tokens = expr.split(";")
+        conditions = {}
+        for token in tokens:
+            subtokens = token.split("=")
+            if len(subtokens) != 2:
+                raise RuntimEerror(f"invalid filter/exclude expression `{expr}`")
+            param_name = subtokens[0].strip()
+            param_conditions = [s.strip() for s in subtokens[1].split(",")]
+            conditions[param_name] = param_conditions
+        return conditions
+    
+    @staticmethod
+    def select_param_point(parameters, conditions:Dict, default:bool=True):
+        selected = {}
+        for param_name in conditions:
+            if param_name not in parameters:
+                selected[param_name] = default
+                continue
+            param_value = parameters[param_name]
+            if isinstance(param_value, float):
+                param_value = round(param_value, 8)
+            param_value = str(param_value)
+            param_conditions = conditions[param_name]
+            selected[param_name] = any([fnmatch.fnmatch(param_value, cond) for cond in param_conditions])
+        selected = all([v for v in selected.values()])
+        return selected
+    
+    def select_param_points(self, param_points:List,
+                            filter_expr:Optional[str]=None,
+                            exclude_expr:Optional[str]=None,
+                            dict_key:Optional=None):
+        selected_param_points = []
+        filter_conditions = self.parse_filter_expr(filter_expr)
+        exclude_conditions = self.parse_filter_expr(exclude_expr)
+        for param_point in param_points:
+            if dict_key:
+                parameters = param_point[dict_key]
+            else:
+                parameters = param_point               
+            keep = True
+            if filter_conditions:
+                keep = self.select_param_point(parameters, filter_conditions, default=True)
+            if exclude_conditions:
+                keep &= not (self.select_param_point(parameters, exclude_conditions, default=False))
+            if keep:
+                selected_param_points.append(param_point)
+        return selected_param_points    
+    
+    def get_param_points(self, dirname:str="",
+                         filter_expr:Optional[str]=None,
+                         exclude_expr:Optional[str]=None):
+        external_param_points = self.get_external_param_points(dirname, filter_expr, exclude_expr)     
+        internal_param_points = self.get_internal_param_points(filter_expr, exclude_expr)
         if len(internal_param_points) == 0:
             internal_param_points = [{}]
         param_points = []
