@@ -19,17 +19,17 @@ def score_distribution_plot(dfs:Dict[str, pd.DataFrame], hist_options:Dict[str,
     styles = parse_styles(plot_styles)
     ax = single_frame(styles=styles, analysis_label_options=analysis_label_options)
     for key in hist_options:
-        processes = hist_options[key]['processes']
+        samples = hist_options[key]['samples']
         hist_style     = hist_options[key].get('style', {})
-        combined_df = pd.concat([dfs[process] for process in processes], ignore_index = True)
+        combined_df = pd.concat([dfs[sample] for sample in samples], ignore_index = True)
         norm_weights = combined_df[weight_name]/combined_df[weight_name].sum()
         y, x, _ = ax.hist(combined_df[score_name], nbins, range=(xmin, xmax), weights=norm_weights, **hist_style,
                           zorder=-5)
     if data_options is not None:
         for key in data_options:
-            processes = data_options[key]['processes']
+            samples = data_options[key]['samples']
             errorbar_style     = data_options[key].get('style', {})
-            combined_df = pd.concat([dfs[process] for process in processes], ignore_index = True)
+            combined_df = pd.concat([dfs[sample] for sample in samples], ignore_index = True)
             norm_weights = combined_df[weight_name]/combined_df[weight_name].sum()
             y, bins = np.histogram(combined_df[score_name], nbins, weights=norm_weights)
             bin_centers  = 0.5*(bins[1:] + bins[:-1])
@@ -59,13 +59,13 @@ def score_distribution_plot_2D(dfs:Dict[str, pd.DataFrame], hist_options:Dict[st
     styles = parse_styles(plot_styles)
     ax = single_frame(styles=styles, analysis_label_options=analysis_label_options)
     key = list(hist_options.keys())[0]
-    processes = hist_options[key]['processes']
+    samples = hist_options[key]['samples']
     hist_style = hist_options[key].get('style_2D', {})
     if hist_style.pop('logz', False):
         log_scale = colors.LogNorm()
     else:
         log_scale = colors.Normalize()
-    combined_df = pd.concat([dfs[process] for process in processes], ignore_index = True)
+    combined_df = pd.concat([dfs[sample] for sample in samples], ignore_index = True)
     norm_weights = combined_df[weight_name]/combined_df[weight_name].sum()
     h, x, y, image = ax.hist2d(x=combined_df[xscore_name], y=combined_df[yscore_name], bins=[xbins, ybins],
                         range=[[xmin, xmax], [ymin, ymax]], weights=norm_weights, **hist_style, 
